﻿using ShipyardContracts.BindingModels;
using ShipyardContracts.SearchModels;
using ShipyardContracts.StoragesContracts;
using ShipyardContracts.ViewModels;
using ShipyardDataBaseImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ShipyardDataBaseImplement.Implements
{
    public class OrderStorage:IOrderStorage
    {
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new ShipyardDataBase();
            var element = context.Orders
                .FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Orders.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }

        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            using var context = new ShipyardDataBase();
            return context.Orders
                .FirstOrDefault(x => model.Id.HasValue && x.Id == model.Id)
                ?.GetViewModel;
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return new();
            }
            using var context = new ShipyardDataBase();
            return context.Orders
                    .Where(x => x.Id == model.Id)
                    .Select(x => x.GetViewModel)
                    .ToList();
        }
        private static OrderViewModel GetViewModel(Order order)
        {
            var viewModel = order.GetViewModel;
            using var context = new ShipyardDataBase();
            var element = context.Ships
                .FirstOrDefault(x => x.Id == order.ShipId);
            viewModel.ShipName = element.ShipName;
            return viewModel;
        }

        public List<OrderViewModel> GetFullList()
        {
            using var context = new ShipyardDataBase();
            return context.Orders
                    .Select(x => new OrderViewModel
                    {
                        Id = x.Id,
                        ShipId = x.ShipId,
                        Count = x.Count,
                        Sum = x.Sum,
                        Status = x.Status,
                        DateCreate = x.DateCreate,
                        DateImplement = x.DateImplement,
                        ShipName = x.Ship.ShipName
                    })
                    .ToList();
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            using var context = new ShipyardDataBase();
            context.Orders.Add(newOrder);
            context.SaveChanges();
            return newOrder.GetViewModel;
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new ShipyardDataBase();
            var order = context.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }
            order.Update(model);
            context.SaveChanges();
            return order.GetViewModel;
        }
    }
}
